
#include <windows.h>
#include <stdio.h>
#include "DevCaps.h"

#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32


HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "My Application"; 

// the rest of the stuff
//......................

BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
	WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}


#define MSG_LEN          1024
#define Display(s)   SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)s)

int        nTabStop = 200;
char       msg[MSG_LEN+1];

HWND       hListBox = NULL;
MMRESULT   rc;

UINT       nDevId = 0;

VOID DisplayDevCaps(HWND hWnd, UINT nDevId);


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   switch( uMsg )
   {
      case WM_CREATE :
              // Create ListBox
              //...............

              hListBox = CreateWindow( "LISTBOX", "",    
                                       WS_CHILD | LBS_NOTIFY | 
                                       WS_VSCROLL | WS_BORDER | 
                                       WS_VISIBLE | LBS_NOINTEGRALHEIGHT |
                                       LBS_USETABSTOPS, 
                                       0, 0, 
                                       0, 0,  
                                       hWnd,              
                                       (HMENU)101,              
                                       hInst,         
                                       NULL );
              SendMessage(hListBox, LB_SETTABSTOPS, 1, (LPARAM)&nTabStop);
              break;

      case WM_SIZE :
              MoveWindow( hListBox, 0, 0, 
                          LOWORD( lParam ), 
                          HIWORD( lParam ), TRUE );
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST:
                        {
                           SendMessage(hListBox, LB_RESETCONTENT, 0, 0);

                           sprintf(msg, "Joystick device driver supports %ld"
                                        " joysticks.", joyGetNumDevs());
                           Display(msg);
                           Display("");

                           for (nDevId = 0; nDevId < joyGetNumDevs(); nDevId++)
                              DisplayDevCaps(hWnd, nDevId);
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;

      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );               
}

/*****************************************************************************
*  DisplayDevCaps
*
*
*
******************************************************************************/

VOID DisplayDevCaps(HWND hWnd, UINT nDevId)
{
   JOYCAPS jc;
   JOYINFO ji;

   Display("");

   // check if joystick is plugged in
   //................................

   if ( joyGetPos(nDevId, &ji) )
   {
       sprintf(msg, "Joystick #%ld:\t Not plugged in.", nDevId);
       Display(msg);
       return;
   }
   else
   {
       sprintf(msg, "Joystick #%ld:\t Capabilities:", nDevId);
       Display(msg);
       Display("");
   }

   joyGetDevCaps(nDevId, &jc, sizeof(JOYCAPS));
   
   sprintf(msg, "Device driver manufacturer ID:\t %ld", jc.wMid);
   Display(msg);
   
   sprintf(msg, "Product ID:\t %ld", jc.wPid);
   Display(msg);
   
   sprintf(msg, "Product name:\t %s", jc.szPname);
   Display(msg);

   sprintf(msg, "Registry key:\t %s", jc.szRegKey);
   Display(msg);

   sprintf(msg, "Joystick driver OEM:\t %s", jc.szOEMVxD);
   Display(msg);
   Display("");
   
   // joystick capabilities
   //......................
   
   sprintf(msg, "Number of buttons:\t Supported:%ld,   In-use:%ld", 
                jc.wMaxButtons, jc.wNumButtons);
   Display(msg);
   
   sprintf(msg, "Number of axes:\t Supported:%ld,   In-use:%ld", 
                jc.wMaxAxes, jc.wNumAxes);
   Display(msg);

   sprintf(msg, "Supports z-coordinate:\t %s", 
           ((jc.wCaps & JOYCAPS_HASZ) ? "Yes" : "No") );
   Display(msg);

   sprintf(msg, "Supports rudder (forth-axis):\t %s", 
           ((jc.wCaps & JOYCAPS_HASR) ? "Yes" : "No") );
   Display(msg);
   
   sprintf(msg, "Supports u-coordinate (fifth-axis):\t %s", 
           ((jc.wCaps & JOYCAPS_HASU) ? "Yes" : "No") );
   Display(msg);
   
   sprintf(msg, "Supports v-coordinate (sixth-axis):\t %s", 
           ((jc.wCaps & JOYCAPS_HASV) ? "Yes" : "No") );
   Display(msg);
   
   sprintf(msg, "Supports point-of-view (POV):\t %s", 
           ((jc.wCaps & JOYCAPS_HASPOV) ? "Yes" : "No") );
   Display(msg);
   
   sprintf(msg, "Supports POV (centered, forward, backward, left,and right):\t %s", 
           ((jc.wCaps & JOYCAPS_POV4DIR) ? "Yes" : "No") );
   Display(msg);
   
   sprintf(msg, "Supports POV degree bearings:\t %s", 
           ((jc.wCaps & JOYCAPS_POVCTS) ? "Yes" : "No") );
   Display(msg);
   
   // range information
   //..................
   
   sprintf(msg, "X-coordinate range (first-axis):\t %ld..%ld", 
                jc.wXmin, jc.wXmax);
   Display(msg);
   
   sprintf(msg, "Y-coordinate range (second-axis):\t %ld..%ld", 
                jc.wYmin, jc.wYmax);
   Display(msg);
   
   sprintf(msg, "Z-coordinate range (third-axis):\t %ld..%ld", 
                jc.wZmin, jc.wZmax);
   Display(msg);
   
   sprintf(msg, "R-coordinate (rudder) range (fourth-axis):\t %ld..%ld", 
                jc.wRmin, jc.wRmax);
   Display(msg);
   
   sprintf(msg, "U-coordinate range (fifth-axis):\t %ld..%ld", 
                jc.wUmin, jc.wUmax);
   Display(msg);
   
   sprintf(msg, "V-coordinate range (sixth-axis):\t %ld..%ld", 
                jc.wVmin, jc.wVmax);
   Display(msg);

   sprintf(msg, "Polling frequency range supported:\t %ld..%ld", 
                jc.wPeriodMin, jc.wPeriodMax);
   Display(msg);

   
}








